/* $Id$ */
/*-
 * Copyright (c) 2003, 2004 Benedikt Meurer <benny@xfce.org>
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __XPREFS_XPREFS_ENGINE_H__
#define __XPREFS_XPREFS_ENGINE_H__

#include <glib.h>

#include <xprefs/xprefs-dbus.h>

G_BEGIN_DECLS;

typedef struct _XPrefsEngine XPrefsEngine;

typedef gpointer XPrefsListenerId;
typedef void (*XPrefsListenerFunc) (XPrefsEngine *engine,
                                    XPrefsListenerId listener_id,
                                    XPrefsEntry *entry,
                                    gpointer user_data);

XPrefsEngine* xprefs_engine_new         (DBusConnection *dbus_conn,
                                         GError **error);
XPrefsEngine* xprefs_engine_ref         (XPrefsEngine *engine);
void          xprefs_engine_unref       (XPrefsEngine *engine);

XPrefsValue*  xprefs_engine_get_value   (XPrefsEngine *engine,
                                         const gchar *key,
                                         GError **error);
gboolean      xprefs_engine_set_value   (XPrefsEngine *engine,
                                         const gchar *key,
                                         const XPrefsValue *value,
                                         GError **error);
gboolean      xprefs_engine_unset       (XPrefsEngine *engine,
                                         const gchar *key,
                                         GError **gerror);

gboolean      xprefs_engine_dir_exists  (XPrefsEngine *engine,
                                         const gchar *key,
                                         GError **gerror);

GList*        xprefs_engine_all_entries (XPrefsEngine *engine,
                                         const gchar *key,
                                         GError **gerror);

GList*        xprefs_engine_all_subdirs (XPrefsEngine *engine,
                                         const gchar *key,
                                         GError **gerror);

gboolean      xprefs_engine_get_bool    (XPrefsEngine *engine,
                                         const gchar *key,
                                         GError **error);
gdouble       xprefs_engine_get_double  (XPrefsEngine *engine,
                                         const gchar *key,
                                         GError **error);
gint32        xprefs_engine_get_int     (XPrefsEngine *engine,
                                         const gchar *key,
                                         GError **error);
gchar*        xprefs_engine_get_string  (XPrefsEngine *engine,
                                         const gchar *key,
                                         GError **error);

gboolean      xprefs_engine_set_bool    (XPrefsEngine *engine,
                                         const gchar *key,
                                         gboolean bool_data,
                                         GError **error);
gboolean      xprefs_engine_set_double  (XPrefsEngine *engine,
                                         const gchar *key,
                                         gdouble double_data,
                                         GError **error);
gboolean      xprefs_engine_set_int     (XPrefsEngine *engine,
                                         const gchar *key,
                                         gint32 int_data,
                                         GError **error);
gboolean      xprefs_engine_set_string  (XPrefsEngine *engine,
                                         const gchar *key,
                                         const gchar *string_data,
                                         GError **error);

XPrefsListenerId  xprefs_engine_listener_add    (XPrefsEngine *engine,
                                                 const gchar *namespace_section,
                                                 XPrefsListenerFunc func,
                                                 gpointer user_data,
                                                 GError **error);
void              xprefs_engine_listener_remove (XPrefsEngine *engine,
                                                 XPrefsListenerId listener_id);

G_END_DECLS

#endif  /* !__XPREFS_XPREFS_ENGINE_H__ */
