/* $Id$ */
/*-
 * Copyright (c) 2003, 2004 Benedikt Meurer <benny@xfce.org>
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <xprefs/xprefs-entry.h>

struct _XPrefsEntry
{
  gchar          *name;
  XPrefsValue    *value;
  gpointer       *user_data;
  GDestroyNotify  dnotify;
};

/**
 **/
XPrefsEntry*
xprefs_entry_new (const gchar *name, const XPrefsValue *value)
{
  return xprefs_entry_new_nocopy (g_strdup (name), xprefs_value_copy (value));
}

/**
 **/
XPrefsEntry*
xprefs_entry_new_nocopy (gchar *name, XPrefsValue *value)
{
  XPrefsEntry *entry;

  g_return_val_if_fail (name != NULL, NULL);

  entry         = g_new0 (XPrefsEntry, 1);
  entry->name   = name;
  entry->value  = value;

  return entry;
}

/**
 **/
void
xprefs_entry_free (XPrefsEntry *entry)
{
  g_return_if_fail (entry != NULL);

  if (G_UNLIKELY (entry->value != NULL))
    xprefs_value_free (entry->value);

  if (entry->dnotify != NULL)
    {
      (*entry->dnotify) (entry->user_data);
    }

  g_free (entry->name);
  g_free (entry);
}

/**
 **/
const gchar*
xprefs_entry_get_name (const XPrefsEntry *entry)
{
  g_return_val_if_fail (entry != NULL, NULL);
  return entry->name;
}

/**
 **/
const XPrefsValue*
xprefs_entry_get_value (const XPrefsEntry *entry)
{
  g_return_val_if_fail (entry != NULL, NULL);
  return entry->value;
}

/**
 **/
void
xprefs_entry_set_value (XPrefsEntry *entry, const XPrefsValue *value)
{
  xprefs_entry_set_value_nocopy (entry, xprefs_value_copy (value));
}

/**
 **/
void
xprefs_entry_set_value_nocopy (XPrefsEntry *entry, XPrefsValue *value)
{
  g_return_if_fail (entry != NULL);
  if (G_LIKELY (entry->value != NULL))
    xprefs_value_free (entry->value);
  entry->value = value;
}

/**
 **/
gpointer
xprefs_entry_get_user_data (const XPrefsEntry *entry)
{
  return entry->user_data;
}

/**
 **/
void
xprefs_entry_set_user_data (XPrefsEntry *entry,
                            gpointer user_data,
                            GDestroyNotify dnotify)
{
  if (entry->dnotify != NULL)
    {
      (*entry->dnotify) (entry->user_data);
    }

  entry->dnotify = dnotify;
  entry->user_data = user_data;
}
