/* $Id$ */
/*-
 * Copyright (c) 2003, 2004 Benedikt Meurer <benny@xfce.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; You may only use version 2 of the License,
 * you have no option to use any other version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <dbus/dbus-glib.h>

#include <xprefs/xprefs.h>

#if 0
static const gchar*
value_to_string (const XPrefsValue *value)
{
  static char buffer[1024];

  switch (value->type)
    {
    case XPREFS_VALUE_BOOL:
      return xprefs_value_get_bool (value) ? "true" : "false";

    case XPREFS_VALUE_DOUBLE:
      g_snprintf (buffer, 1024, "%g", xprefs_value_get_double (value));
      break;

    case XPREFS_VALUE_INT:
      g_snprintf (buffer, 1024, "%d", xprefs_value_get_int (value));
      break;

    case XPREFS_VALUE_STRING:
      return xprefs_value_get_string (value);

    default:
      g_assert_not_reached ();
    }

  return buffer;
}
#endif

int
main (int argc, char *argv[])
{
  DBusConnection *connection;
  XPrefsEngine *engine;
  GMainLoop *loop;
  GError *gerror;

  _malloc_options = "AJV";
  gerror = NULL;

  g_type_init ();

  loop = g_main_loop_new (NULL, FALSE);

  /* open connection to D-BUS message daemon */
  connection = dbus_bus_get_with_g_main (DBUS_BUS_SESSION, &gerror);
  if (connection == NULL)
    {
      g_printerr ("Unable to connect to D-BUS bus message daemon: %s\n",
                  gerror->message);
      g_error_free (gerror);
      return EXIT_FAILURE;
    }

  /* connect to the XPrefs database service */
  engine = xprefs_engine_new (connection, &gerror);
  if (engine == NULL)
    {
      g_printerr ("Unable to connect to XPrefs database service: %s\n",
                  gerror->message);
      g_error_free (gerror);
      return EXIT_FAILURE;
    }

  if (argc == 2)
    {
      const gchar *key = argv[1];
      XPrefsValue *value;

      value = xprefs_engine_get_value (engine, key, &gerror);

      if (value == NULL)
        {
          g_printerr ("Unable to get value for key '%s': %s\n",
                      key, gerror->message);
          g_error_free (gerror);
          return EXIT_FAILURE;
        }

      g_print ("Key %s = %s\n", key, xprefs_value_to_string (value));

      xprefs_value_free (value);
    }
  else if (argc == 3)
    {
      const gchar *key = argv[1];
      XPrefsValue *value;
      gboolean result;

      value = xprefs_value_new (XPREFS_VALUE_STRING);
      xprefs_value_set_string (value, argv[2]);

      result = xprefs_engine_set_value (engine, key, value, &gerror); 
      if (!result)
        {
          g_printerr ("Unable to set value for key '%s': %s\n",
                      key, gerror->message);
          g_error_free (gerror);
          return EXIT_FAILURE;
        }

      xprefs_value_free (value);
    }
  else
    {
      g_printerr ("Invalid parameters\n");
      return EXIT_FAILURE;
    }

  xprefs_engine_unref (engine);

  return EXIT_SUCCESS;
}

